# -*- coding: utf-8 -*-

# Curlew - Easy to use multimedia converter
#
# Copyright (C) 2012-2016 Fayssal Chamekh <chamfay@gmail.com>
#
# Released under terms on waqf public license.
#
# Curlew is free software; you can redistribute it and/or modify it 
# under the terms of the latest version waqf public license as published by 
# ojuba.org.
#
# Curlew is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty 
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#        
# The latest version of the license can be found on:
# http://www.ojuba.org/wiki/doku.php/waqf/license


LANGUAGES = {
             'العربية': 'ar',
             'Português - Brazil': 'br',
             'Český': 'cs',
             'English': 'en',
             'Español': 'es',
             'Euskara': 'eu',
             'Français': 'fr',
             'Italiano': 'it',
             'Nederlands': 'nl',
             'Polski': 'pl',
             'Русский': 'ru',
             'Srpski': 'sr',
             'Traditional Chinese: 正體中文': 'zh_TW'
             }